# coding=utf8
""" Brain REST

Handles starting the REST server using the Brain service
"""

__author__		= "Chris Nasr"
__version__		= "1.0.0"
__copyright__	= "Ouroboros Coding Inc."
__email__		= "chris@ouroboroscoding.com"
__created__		= "2022-08-25"

# Ouroboros imports
from body import errors
from config import config

# Python imports
from sys import stderr

# Pip imports
from google_auth_oauthlib.flow import Flow
from RestOC import EMail, REST, Services, Session

# Module imports
from brain.service import Brain

# Global
_ogFlow = None

def google_login():
	"""Google OAuth Login

	Handles request to show the google login using oauth

	Returns
		str
	"""

	# Pull in the flow variable
	global _ogFlow

	# If we have no flow
	if not _ogFlow:
		return REST.bottle.abort(401, 'Google OAuth not initialised')

	# Get the auth url from google
	sAuthURL, _ = _ogFlow.authorization_url(
		prompt = 'consent'
	)

	# Redirect the user to the login
	return REST.bottle.redirect(sAuthURL)

def run(conf: dict):
	"""Run

	Starts the http REST server

	Arguments:
		conf (dict): The brain config

	Returns:
		None
	"""

	# Init the email module
	EMail.init(config.email({
		'error_to': 'errors@localhost',
		'from': 'admin@localhost',
		'smtp': {
			'host': 'localhost',
			'port': 587,
			'tls': True,
			'user': 'noone',
			'passwd': 'nopasswd'
		}
	}))

	# Init the Session module
	Session.init('session')

	# Get the REST config
	dRest = config.rest({
		'allowed': 'localhost',
		'default': {
			'domain': 'localhost',
			'host': '0.0.0.0',
			'port': 8800,
			'protocol': 'http',
			'workers': 1
		},
		'services': {
			'brain': {'port': 0},
			'mouth': {'port': 1}
		}
	})

	# Create the REST config instance
	oRestConf = REST.Config(dRest)

	# If google oauth is enabled
	if conf['google'] is not False:

		# If the redirect is missing
		if 'redirect' in conf['google'] and \
			isinstance(conf['google']['redirect'], str):

			# Pull in Flow instance
			global _ogFlow

			# Init scopes
			lScopes = [
				'openid',
				'https://www.googleapis.com/auth/userinfo.email',
				'https://www.googleapis.com/auth/userinfo.profile'
			]

			# If we got a string
			if isinstance(conf['google']['client_secret'], str):

				# Assume the string is a client secrets file
				_ogFlow = Flow.from_client_secrets_file(
					conf['google']['client_secret'],
					scopes = lScopes,
					redirect_uri = conf['google']['redirect']
				)

			# Else if we got a config
			elif isinstance(conf['google']['client_secret'], dict):

				# Assume the dict is a client config
				_ogFlow = Flow.from_client_config(
					conf['google']['client_secret'],
					scopes = lScopes,
					redirect_uri = conf['google']['redirect']
				)

			# Else
			else:
				print(
					'brain.google.client_secret must be str (filename) ' \
					'or dict (config)',
					file = stderr
				)

		# Else
		else:
			print('brain.google.redirect must be str', file = stderr)

	# Set verbose mode if requested
	if conf['verbose']:
		Services.verbose()

	# Get all the services
	dServices = { k: None for k in dRest['services'] }

	# Add this service
	dServices['brain'] = Brain(conf, _ogFlow)

	# Register all services
	Services.register(
		dServices,
		oRestConf,
		config.services.salt(),
		config.services.internal_key_timeout(10)
	)

	# Create the HTTP server and map requests to service
	oServer = REST.Server({

		'/google/auth': { 'methods': REST.CREATE },

		'/passwd/verify': { 'methods': REST.CREATE },

		'/permissions': { 'methods': REST.READ | REST.UPDATE },
		'/permissions/add': { 'methods': REST.CREATE },

		'/search': { 'methods': REST.READ },

		'/session': { 'methods': REST.READ },

		'/signin': { 'methods': REST.POST },
		'/signin/to': { 'methods': REST.POST },
		'/signout': { 'methods': REST.POST },
		'/signup': { 'methods': REST.POST },

		'/user': { 'methods': REST.CREATE | REST.READ | REST.UPDATE },
		'/user/email': { 'methods': REST.UPDATE },
		'/user/email/verify': { 'methods': REST.UPDATE },
		'/user/names': { 'methods': REST.READ },
		'/user/passwd': { 'methods': REST.UPDATE },
		'/user/passwd/forgot': { 'methods': REST.CREATE | REST.UPDATE },
		'/user/setup': { 'methods': REST.READ | REST.UPDATE },
		'/user/setup/key': { 'methods': REST.READ },
		'/user/setup/send': { 'methods': REST.CREATE },
		'/users/by/email': { 'methods': REST.READ },
		'/users/by/id': { 'method': REST.READ },

		'/verify': { 'methods': REST.READ },

		},
		'brain',
		'https?://(.*\\.)?%s' % config.rest.allowed('localhost').replace('.', '\\.'),
		error_callback=errors.service_error
	)

	# If we have a flow
	if _ogFlow:

		# Bind the google login uri
		oServer.route('/google/login', 'GET', google_login)

	# Run the server
	oServer.run(
		host = oRestConf['brain']['host'],
		port = oRestConf['brain']['port'],
		workers = oRestConf['brain']['workers'],
		timeout = 'timeout' in oRestConf['brain'] and \
					oRestConf['brain']['timeout'] or \
					30
	)